@@ -35,18 +35,23 @@
                 forward requirements.
    08 Jul 2004: Revised logic in lu6chk based on new code from
                 Michael Saunders.
+   01 Dec 2005: Add support for CMEX interface (disable by undef MATLAB)
+                Also include various bug fixes (disable by undef YZHANG)
+                Yin Zhang <yzhang@cs.utexas.edu>
+   01 Jan 2006: Added storage of singular indeces, not only the last.
    ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <stdarg.h>
-/* #include <varargs.h>  For UNIX 5 compatibility */
 #include <string.h>
 #include <float.h>
 #include <math.h>
 #include "lusol.h"
 #include "myblas.h"
-
+#ifdef MATLAB
+  #include "mex.h"
+#endif
 
 /* LUSOL Object creation and destruction */
 
@@ -54,7 +59,7 @@ void *clean_realloc(void *oldptr, int width, int newsize, int oldsize)
 {
   newsize *= width;
   oldsize *= width;
-  oldptr = realloc(oldptr, newsize);
+  oldptr = LUSOL_REALLOC(oldptr, newsize);
   if(newsize > oldsize)
 /*    MEMCLEAR(oldptr+oldsize, newsize-oldsize); */
     memset((char *)oldptr+oldsize, '\0', newsize-oldsize);
@@ -240,7 +245,7 @@ LUSOLrec *LUSOL_create(FILE *outstream, int msgfil, int pivotmodel, int updateli
 {
   LUSOLrec *newLU;
 
-  newLU = (LUSOLrec *) calloc(1, sizeof(*newLU));
+  newLU = (LUSOLrec *) LUSOL_CALLOC(1, sizeof(*newLU));
   if(newLU == NULL)
     return( newLU );
 
@@ -266,7 +271,7 @@ LUSOLrec *LUSOL_create(FILE *outstream, int msgfil, int pivotmodel, int updateli
 
   newLU->parmlu[LUSOL_RP_SMARTRATIO]       = LUSOL_DEFAULT_SMARTRATIO;
 #ifdef ForceRowBasedL0
-  newLU->luparm[LUSOL_IP_USEROWL0]         = LUSOL_BASEORDER;
+  newLU->luparm[LUSOL_IP_ACCELERATION]     = LUSOL_BASEORDER;
 #endif
   newLU->luparm[LUSOL_IP_KEEPLU]           = TRUE;
   newLU->luparm[LUSOL_IP_UPDATELIMIT]      = updatelimit;
@@ -278,6 +283,8 @@ LUSOLrec *LUSOL_create(FILE *outstream, int msgfil, int pivotmodel, int updateli
 
 MYBOOL LUSOL_sizeto(LUSOLrec *LUSOL, int init_r, int init_c, int init_a)
 {
+  if(init_c == 0)
+    LUSOL_FREE(LUSOL->isingular);
   if(LUSOL_realloc_a(LUSOL, init_a) &&
      LUSOL_realloc_r(LUSOL, init_r) &&
      LUSOL_realloc_c(LUSOL, init_c))
@@ -330,9 +337,13 @@ void LUSOL_setpivotmodel(LUSOLrec *LUSOL, int pivotmodel, int initlevel)
     newFM = 5.0;
     newUM = newFM / 2;
   }
-  else {                                      /* Very tight pivot tolerances for extra accuracy */
+  else if(initlevel == LUSOL_PIVTOL_SUPER) {  /* Very tight pivot tolerances for extra accuracy */
     newFM = 2.5;
-    newUM = newFM / 2;
+    newUM = 1.99;
+  }
+  else {                                      /* Extremely tight pivot tolerances for extra accuracy */
+    newFM = 1.99;
+    newUM = newFM / 1.49;
   }
 
   /* Set the tolerances */
@@ -342,10 +353,12 @@ void LUSOL_setpivotmodel(LUSOLrec *LUSOL, int pivotmodel, int initlevel)
 
 MYBOOL LUSOL_tightenpivot(LUSOLrec *LUSOL)
 {
-  /* REAL newvalue; */
+#if 0
+  REAL newvalue;
+#endif
 
   /* Give up tightening if we are already less than limit and we cannot change strategy */
-  if(MIN(LUSOL->parmlu[LUSOL_RP_UPDATEMAX_Lij],
+  if(MIN(LUSOL->parmlu[LUSOL_RP_FACTORMAX_Lij],
          LUSOL->parmlu[LUSOL_RP_UPDATEMAX_Lij]) < 1.1) {
     if(LUSOL->luparm[LUSOL_IP_PIVOTTYPE] >= LUSOL_PIVMOD_TRP)
       return( FALSE );
@@ -357,8 +370,7 @@ MYBOOL LUSOL_tightenpivot(LUSOLrec *LUSOL)
 #if 0   /* This is Michael Saunder's proposed tightening procedure */
   newvalue = sqrt(LUSOL->parmlu[LUSOL_RP_FACTORMAX_Lij]);
   LUSOL->parmlu[LUSOL_RP_FACTORMAX_Lij] = newvalue;
-  LUSOL->parmlu[LUSOL_RP_UPDATEMAX_Lij] = MIN(newvalue,
-                                              LUSOL->parmlu[LUSOL_RP_UPDATEMAX_Lij]);
+  SETMIN(LUSOL->parmlu[LUSOL_RP_UPDATEMAX_Lij], newvalue);
 #elif 0 /* This is Kjell Eikland's schedule #1 */
   newvalue = sqrt(LUSOL->parmlu[LUSOL_RP_FACTORMAX_Lij]);
   LUSOL->parmlu[LUSOL_RP_FACTORMAX_Lij] = newvalue;
@@ -370,6 +382,75 @@ MYBOOL LUSOL_tightenpivot(LUSOLrec *LUSOL)
   return( TRUE );
 }
 
+MYBOOL LUSOL_addSingularity(LUSOLrec *LUSOL, int singcol, int *inform)
+{
+  int NSING = LUSOL->luparm[LUSOL_IP_SINGULARITIES],
+      ASING = LUSOL->luparm[LUSOL_IP_SINGULARLISTSIZE];
+
+  /* Check if we need to allocated list memory to store multiple singularities */
+  if((NSING > 0) && (NSING >= ASING)) {
+
+    /* Increase list in "reasonable" steps */
+    ASING += (int) (10.0 * (log10((REAL) LUSOL->m)+1.0));
+    LUSOL->isingular = (int *) LUSOL_REALLOC(LUSOL->isingular, sizeof(*LUSOL->isingular)*(ASING+1));
+    if(LUSOL->isingular == NULL) {
+      LUSOL->luparm[LUSOL_IP_SINGULARLISTSIZE] = 0;
+      *inform = LUSOL_INFORM_NOMEMLEFT;
+      return( FALSE );
+    }
+    LUSOL->luparm[LUSOL_IP_SINGULARLISTSIZE] = ASING;
+
+    /* Transfer the first singularity if the list was just created */
+    if(NSING == 1)
+      LUSOL->isingular[NSING] = LUSOL->luparm[LUSOL_IP_SINGULARINDEX];
+  }
+
+  /* Update singularity count and store its index */
+  NSING++;
+  if(NSING > 1) {
+    LUSOL->isingular[0] = NSING;
+    LUSOL->isingular[NSING] = singcol;
+  }
+  LUSOL->luparm[LUSOL_IP_SINGULARITIES] = NSING;
+
+  /* Mimic old logic by keeping the last singularity stored */
+  LUSOL->luparm[LUSOL_IP_SINGULARINDEX] = singcol;
+
+  return( TRUE );
+}
+
+int LUSOL_getSingularity(LUSOLrec *LUSOL, int singitem)
+{
+  if((singitem > LUSOL->luparm[LUSOL_IP_SINGULARITIES]) || (singitem < 0))
+    singitem = -1;
+  else if(singitem == 0)
+    singitem = LUSOL->luparm[LUSOL_IP_SINGULARITIES];
+  else if(singitem > 1)
+    singitem = LUSOL->isingular[singitem];
+  else
+    singitem = LUSOL->luparm[LUSOL_IP_SINGULARINDEX];
+  return( singitem );
+}
+
+int LUSOL_findSingularityPosition(LUSOLrec *LUSOL, int singcol)
+/* The purpose of this routine is to find the slack row/column in
+   user-index that was singular in the last unsuccessful column
+   update; zero is returned if the search was unsuccessful.
+   By adding a slack at this position this particular singularity
+   should disappear.
+   (Source: Michael A. Saunders; private communication to KE) */
+{
+#if 0 /* Michael Saunders version */
+  int j;
+  for(j = LUSOL->m; j > 0; j--)
+    if(LUSOL->iq[j] == singcol)
+      break;
+  singcol = j;
+#else /* Kjell Eikland version (note that iqinv could be invalid in early versions of LUSOL) */
+  singcol = LUSOL->iqinv[singcol];
+#endif
+  return( LUSOL->ip[singcol] );
+}
 
 char *LUSOL_informstr(LUSOLrec *LUSOL, int inform)
 {
@@ -380,14 +461,14 @@ char *LUSOL_informstr(LUSOLrec *LUSOL, int inform)
    "LUSOL_LUUNSTABLE: Unstable factorization",
    "LUSOL_ADIMERR: Row or column count exceeded",
    "LUSOL_ADUPLICATE: Duplicate A matrix entry found",
-   "",
-   "",
+   "(Undefined message)",
+   "(Undefined message)",
    "LUSOL_ANEEDMEM: Insufficient memory for factorization",
    "LUSOL_FATALERR: Fatal internal error",
    "LUSOL_NOPIVOT: Found no suitable pivot",
    "LUSOL_NOMEMLEFT: Could not obtain more memory"};
-  if(inform < LUSOL_INFORM_MIN || inform > LUSOL_INFORM_MAX)
-    inform = /* LUSOL->luparm[LUSOL_IP_INFORM] */ LUSOL_INFORM_FATALERR;
+  if((inform < LUSOL_INFORM_MIN) || (inform > LUSOL_INFORM_MAX))
+    inform = LUSOL->luparm[LUSOL_IP_INFORM];
   return(informText[inform-LUSOL_INFORM_MIN]);
 }
 
@@ -444,7 +525,6 @@ void LUSOL_clear(LUSOLrec *LUSOL, MYBOOL nzonly)
   }
 }
 
-
 MYBOOL LUSOL_assign(LUSOLrec *LUSOL, int iA[], int jA[], REAL Aij[], int nzcount, MYBOOL istriplet)
 {
   int k, m, n, ij, kol;
@@ -532,32 +612,38 @@ void LUSOL_free(LUSOLrec *LUSOL)
   LUSOL_realloc_c(LUSOL, 0);
   if(LUSOL->L0 != NULL)
     LUSOL_matfree(&(LUSOL->L0));
+  if(LUSOL->U != NULL)
+    LUSOL_matfree(&(LUSOL->U));
   if(!is_nativeBLAS())
     unload_BLAS();
-  free(LUSOL);
+  LUSOL_FREE(LUSOL);
 }
 
 void LUSOL_report(LUSOLrec *LUSOL, int msglevel, char *format, ...)
 {
   va_list ap;
 
-  va_start(ap, format);
   if(LUSOL == NULL) {
+    va_start(ap, format);
     vfprintf(stderr, format, ap);
+    va_end(ap);
   }
   else if(msglevel >= 0  /*LUSOL->luparm[2]*/) {
     if(LUSOL->writelog != NULL) {
       char buff[255];
 
+      va_start(ap, format);
       vsprintf(buff, format, ap);
+      va_end(ap);
       LUSOL->writelog(LUSOL, LUSOL->loghandle, buff);
     }
     if(LUSOL->outstream != NULL) {
+      va_start(ap, format);
       vfprintf(LUSOL->outstream, format, ap);
+      va_end(ap);
       fflush(LUSOL->outstream);
     }
   }
-  va_end(ap);
 }
 
 void LUSOL_timer(LUSOLrec *LUSOL, int timerid, char *text)
@@ -627,24 +713,6 @@ int LUSOL_replaceColumn(LUSOLrec *LUSOL, int jcol, REAL v[])
   return( inform );
 }
 
-int LUSOL_findColumnPosition(LUSOLrec *LUSOL, int jcol)
-/* The purpose of this routine is to find the slack row/column in
-   user-index that was singular in the last unsuccessful column
-   update; zero is returned if the search was unsuccessful.
-   (Source: Michael A. Saunders; private communication to KE) */
-{
-  int j;
-
-#if 1 /* Michael Saunders version */
-  for(j = LUSOL->m; j > 0; j--)
-    if(LUSOL->iq[j] == jcol)
-      break;
-#else /* Kjell Eikland version (note that iqinv could be invalid) */
-  j = LUSOL->iqinv[jcol];
-#endif
-  return( j );
-}
-
 REAL LUSOL_vecdensity(LUSOLrec *LUSOL, REAL V[])
 {
   int I, N = 0;
@@ -706,13 +774,13 @@ LUSOLmat *LUSOL_matcreate(int dim, int nz)
 {
   LUSOLmat *newm;
 
-  newm = (LUSOLmat *) calloc(1, sizeof(*newm));
+  newm = (LUSOLmat *) LUSOL_CALLOC(1, sizeof(*newm));
   if(newm != NULL) {
-    newm->a    = (REAL *) malloc((nz+1)*sizeof(REAL));
-    newm->lenx = (int *)  malloc((dim+1)*sizeof(int));
-    newm->indx = (int *)  malloc((dim+1)*sizeof(int));
-    newm->indr = (int *)  malloc((nz+1)*sizeof(int));
-    newm->indc = (int *)  malloc((nz+1)*sizeof(int));
+    newm->a    = (REAL *) LUSOL_MALLOC((nz+1)*sizeof(REAL));
+    newm->lenx = (int *)  LUSOL_MALLOC((dim+1)*sizeof(int));
+    newm->indx = (int *)  LUSOL_MALLOC((dim+1)*sizeof(int));
+    newm->indr = (int *)  LUSOL_MALLOC((nz+1)*sizeof(int));
+    newm->indc = (int *)  LUSOL_MALLOC((nz+1)*sizeof(int));
     if((newm->a == NULL) ||
        (newm->lenx == NULL) || (newm->indx == NULL) ||
        (newm->indr == NULL) || (newm->indc == NULL))
@@ -724,11 +792,11 @@ void LUSOL_matfree(LUSOLmat **mat)
 {
   if((mat == NULL) || (*mat == NULL))
     return;
-  FREE((*mat)->a);
-  FREE((*mat)->indc);
-  FREE((*mat)->indr);
-  FREE((*mat)->lenx);
-  FREE((*mat)->indx);
-  FREE(*mat);
+  LUSOL_FREE((*mat)->a);
+  LUSOL_FREE((*mat)->indc);
+  LUSOL_FREE((*mat)->indr);
+  LUSOL_FREE((*mat)->lenx);
+  LUSOL_FREE((*mat)->indx);
+  LUSOL_FREE(*mat);
 }
 
