@@ -182,19 +182,33 @@ STATIC MYBOOL stallMonitor_check(lprec *lp, int rownr, int colnr, int lastnr,
   /* Also require that we have a measure of infeasibility-stalling */
   if(isStalled) {
     REAL testvalue, refvalue = monitor->epsvalue;
+#if 1
     if(monitor->isdual)
       refvalue *= 1000*log10(9.0+lp->rows);
     else
       refvalue *= 1000*log10(9.0+lp->columns);
+#else
+      refvalue *= 1000*log10(9.0+lp->sum);
+#endif
     testvalue = my_reldiff(monitor->thisinfeas, monitor->previnfeas);
     isStalled &= (fabs(testvalue) < refvalue);
 
     /* Check if we should force "major" pivoting, i.e. no bound flips;
       this is activated when we see the feasibility deteriorate */
 /*    if(!isStalled && (testvalue > 0) && (TRUE || is_action(lp->anti_degen, ANTIDEGEN_BOUNDFLIP))) */
+#if !defined _PRICE_NOBOUNDFLIP
     if(!isStalled && (testvalue > 0) && is_action(lp->anti_degen, ANTIDEGEN_BOUNDFLIP))
       acceptance = AUTOMATIC;
   }
+#else
+    if(!isStalled && (testvalue > 0) && !ISMASKSET(lp->piv_strategy, PRICE_NOBOUNDFLIP)) {
+      SETMASK(lp->piv_strategy, PRICE_NOBOUNDFLIP);
+      acceptance = AUTOMATIC;
+    }
+  }
+  else
+    CLEARMASK(lp->piv_strategy, PRICE_NOBOUNDFLIP);
+#endif
 
 #if 1
   isCreeping = FALSE;
@@ -652,6 +666,10 @@ STATIC int primloop(lprec *lp, MYBOOL primalfeasible, REAL primaloffset)
 
    /* Find best column to enter the basis */
 RetryCol:
+#if 0
+    if(verify_solution(lp, FALSE, "spx_loop") > 0)
+      i = 1; /* This is just a debug trap */
+#endif
     if(!changedphase) {
       i = 0;
       do {
@@ -854,7 +872,7 @@ Optimality:
 
       /* Check if we are still primal feasible; the default assumes that this check
          is not necessary after the relaxed problem has been solved satisfactorily. */
-      if((lp->bb_level <= 1) || (lp->improve & IMPROVE_BBSIMPLEX)) {
+      if((lp->bb_level <= 1) || (lp->improve & IMPROVE_BBSIMPLEX) /* || (lp->bb_rule & NODE_RCOSTFIXING) */) { /* NODE_RCOSTFIXING fix */
         set_action(&lp->piv_strategy, PRICE_FORCEFULL);
           i = rowdual(lp, lp->rhs, FALSE, FALSE, NULL);
         clear_action(&lp->piv_strategy, PRICE_FORCEFULL);
@@ -1386,7 +1404,7 @@ RetryRow:
         /* Check if we are still dual feasible; the default assumes that this check
           is not necessary after the relaxed problem has been solved satisfactorily. */
         colnr = 0;
-        if((dualoffset != 0) || (lp->bb_level <= 1) || (lp->improve & IMPROVE_BBSIMPLEX)) {
+        if((dualoffset != 0) || (lp->bb_level <= 1) || (lp->improve & IMPROVE_BBSIMPLEX) || (lp->bb_rule & NODE_RCOSTFIXING)) { /* NODE_RCOSTFIXING fix */
           set_action(&lp->piv_strategy, PRICE_FORCEFULL);
             colnr = colprim(lp, drow, nzdrow, FALSE, 1, &candidatecount, FALSE, NULL);
           clear_action(&lp->piv_strategy, PRICE_FORCEFULL);
@@ -2031,7 +2049,11 @@ STATIC int spx_solve(lprec *lp)
     lp->bfp_restart(lp);
 
   lp->spx_status = presolve(lp);
-  if(lp->spx_status != RUNNING)
+  if(lp->spx_status == PRESOLVED) {
+    status = lp->spx_status;
+    goto Reconstruct;
+  }
+  else if(lp->spx_status != RUNNING)
     goto Leave;
 
   iprocessed = !lp->wasPreprocessed;
@@ -2055,6 +2077,7 @@ STATIC int spx_solve(lprec *lp)
       postprocess(lp);
 
     /* Restore data related to presolve (mainly a placeholder as of v5.1) */
+Reconstruct:
     if(!postsolve(lp, status))
       report(lp, SEVERE, "spx_solve: Failure during postsolve.\n");
 
@@ -2158,7 +2181,26 @@ int lin_solve(lprec *lp)
   /* Reset heuristic in preparation for next run (if any) */
   lp->bb_heuristicOF = my_chsign(is_maxim(lp), lp->infinite);
 
+  /* Check that correct status code is returned */
+/*
+   peno 26.12.07
+   status was not set to SUBOPTIMAL, only lp->spx_status
+   Bug occured by a change in 5.5.0.10 when  && (lp->bb_totalnodes > 0) was added
+   added status =
+   See UnitTest3
+*/
+/*
+   peno 12.01.08
+   If an integer solution is found with the same objective value as the relaxed solution then
+   searching is stopped. This by setting lp->bb_break. However this resulted in a report of SUBOPTIMAL
+   solution. For this,  && !bb_better(lp, OF_DUALLIMIT, OF_TEST_BE) is added in the test.
+   See UnitTest20
+*/
+  if((lp->spx_status == OPTIMAL) && (lp->bb_totalnodes > 0)) {
+    if((lp->bb_break && !bb_better(lp, OF_DUALLIMIT, OF_TEST_BE)) /* ||
+       ISMASKSET(lp->trace, TRACE_NOBBLIMIT) */)
+    status = lp->spx_status = SUBOPTIMAL;
+  }
+
   return( status );
 }
-
-
