@@ -1,6 +1,7 @@
 #include <string.h>
 #include <time.h>
 #include <signal.h>
+#include <math.h>
 #include "lp_lib.h"
 
 #ifdef FORTIFY
@@ -40,18 +41,21 @@ void print_help(char *argv[])
 #endif
   printf("-mps\t\tread from MPS file in fixed format\n");
   printf("-fmps\t\tread from MPS file in free format\n");
+  printf("-rpar filename\tread parameters from filename.\n");
+  printf("-rparopt options\n\t\toptions for parameter file:\n");
+  printf("\t\t -H headername: header name for parameters. By default 'Default'\n");
   printf("-rxli xliname filename\n\t\tread file with xli library\n");
   printf("-rxlidata datafilename\n\t\tdata file name for xli library.\n");
   printf("-rxliopt options\n\t\toptions for xli library.\n");
   printf("-rbas filename\tread basis from filename.\n");
-  printf("-rpar filename\tread parameters from filename.\n");
-  printf("-rparopt options\n\t\toptions for parameter file:\n");
-  printf("\t\t -H headername: header name for parameters. By default 'Default'\n");
+  printf("-gbas filename\tguess basis with variables from filename.\n");
   printf("-wlp filename\twrite to LP file\n");
   printf("-wmps filename\twrite to MPS file in fixed format\n");
   printf("-wfmps filename\twrite to MPS file in free format\n");
   printf("-wxli xliname filename\n\t\twrite file with xli library\n");
   printf("-wxliopt options\n\t\toptions for xli library.\n");
+  printf("-wxlisol xliname filename\n\t\twrite solution file with xli library\n");
+  printf("-wxlisolopt options\n\t\toptions for xli library.\n");
   printf("-wbas filename\twrite basis to filename.\n");
   printf("-wpar filename\twrite parameters to filename.\n");
   printf("-wparopt options\n\t\toptions for parameter file:\n");
@@ -84,6 +88,8 @@ void print_help(char *argv[])
   printf("-pivla\t\tScan entering/leaving columns alternatingly left/right.\n");
   printf("-pivh\t\tUse Harris' primal pivot logic rather than the default.\n");
   printf("-pivt\t\tUse true norms for Devex and Steepest Edge initializations.\n");
+  printf("-o0\t\tDon't put objective in basis%s.\n", DEF_OBJINBASIS ? "" : " (default)");
+  printf("-o1\t\tPut objective in basis%s.\n", DEF_OBJINBASIS ? " (default)" : "");
   printf("-s <mode> <scaleloop>\tuse automatic problem scaling.\n");
   printf("\t -s0: No scaling\n");
   printf("\t -s1: Geometric scaling (default)\n");
@@ -268,7 +274,7 @@ static void __WINAPI LPMessageCB(lprec *lp, void *USERHANDLE, int msg)
     DoReport(lp, "Improved MILP ");
 }
 
-void write_model(lprec *lp, char *wlp, char *wmps, char *wfmps, char *wxli, char *wxliname, char *wxlioptions)
+void write_model(lprec *lp, char *wlp, char *wmps, char *wfmps, char *wxli, char *wxlisol, char *wxliname, char *wxlioptions)
 {
   if(wlp != NULL)
     write_lp(lp, wlp);
@@ -279,7 +285,7 @@ void write_model(lprec *lp, char *wlp, char *wmps, char *wfmps, char *wxli, char
   if(wfmps != NULL)
     write_freemps(lp, wfmps);
 
-  if(wxli != NULL) {
+  if((wxliname != NULL) && (wxli != NULL)) {
     if(!set_XLI(lp, wxliname)) {
       fprintf(stderr, "Unable to set XLI library (%s).\n", wxliname);
       EndOfPgr(FORCED_EXIT);
@@ -287,6 +293,15 @@ void write_model(lprec *lp, char *wlp, char *wmps, char *wfmps, char *wxli, char
     write_XLI(lp, wxli, wxlioptions, FALSE);
     set_XLI(lp, NULL);
   }
+
+  if((wxliname != NULL) && (wxlisol != NULL)) {
+    if(!set_XLI(lp, wxliname)) {
+      fprintf(stderr, "Unable to set XLI library (%s).\n", wxliname);
+      EndOfPgr(FORCED_EXIT);
+    }
+    write_XLI(lp, wxlisol, wxlioptions, TRUE);
+    set_XLI(lp, NULL);
+  }
 }
 
 static void or_value(int *value, int orvalue)
@@ -339,6 +354,7 @@ int main(int argc, char *argv[])
   int scalemode1 = -1;
   int scalemode2 = -1;
   int crashmode = -1;
+  char *guessbasis = NULL;
   /* short timeoutok = FALSE; */
   long sectimeout = -1;
   int result;
@@ -358,7 +374,7 @@ int main(int argc, char *argv[])
   REAL break_at_value = 0;
   FILE *fpin = stdin;
   char *bfp = NULL;
-  char *rxliname = NULL, *rxli = NULL, *rxlidata = NULL, *rxlioptions = NULL, *wxliname = NULL, *wxli = NULL, *wxlioptions = NULL;
+  char *rxliname = NULL, *rxli = NULL, *rxlidata = NULL, *rxlioptions = NULL, *wxliname = NULL, *wxlisol = NULL, *wxli = NULL, *wxlioptions = NULL, *wxlisoloptions = NULL;
   char *rbasname = NULL, *wbasname = NULL;
   char *debugdump_before = NULL;
   char *debugdump_after = NULL;
@@ -366,6 +382,8 @@ int main(int argc, char *argv[])
   char *rparoptions = NULL;
   char *wparname = NULL;
   char *wparoptions = NULL;
+  char obj_in_basis = -1;
+  MYBOOL ok;
 # define SCALINGTHRESHOLD 0.03
 
   /* read command line arguments */
@@ -375,6 +393,7 @@ int main(int argc, char *argv[])
 # endif
 
   for(i = 1; i < argc; i++) {
+    ok = FALSE;
     if(strncmp(argv[i], "-v", 2) == 0) {
       if (argv[i][2])
         verbose = atoi(argv[i] + 2);
@@ -418,7 +437,7 @@ int main(int argc, char *argv[])
     else if(strcmp(argv[i], "-Br") == 0)
       or_value(&bb_rule2, NODE_RANDOMIZEMODE);
     else if(strcmp(argv[i], "-BG") == 0)
-      or_value(&bb_rule2, NODE_GUBMODE);
+      or_value(&bb_rule2, 0 /* NODE_GUBMODE */); /* doesn't work yet */
     else if(strcmp(argv[i], "-Bd") == 0)
       or_value(&bb_rule2, NODE_DYNAMICMODE);
     else if(strcmp(argv[i], "-Bs") == 0)
@@ -515,6 +534,9 @@ int main(int argc, char *argv[])
         case 6:
 	  set_value(&scalemode1, SCALE_RANGE);
 	  break;
+	case 7:
+	  set_value(&scalemode1, SCALE_MEAN | SCALE_QUADRATIC);
+	  break;
         }
       }
       else
@@ -524,6 +546,8 @@ int main(int argc, char *argv[])
     }
     else if(strncmp(argv[i], "-C", 2) == 0)
       crashmode = atoi(argv[i] + 2);
+	else if((strcmp(argv[i],"-gbas") == 0) && (i + 1 < argc))
+      guessbasis = argv[++i];
     else if(strcmp(argv[i], "-t") == 0)
       tracing = TRUE;
     else if(strncmp(argv[i], "-S", 2) == 0) {
@@ -628,6 +652,12 @@ int main(int argc, char *argv[])
     }
     else if((strcmp(argv[i],"-wxliopt") == 0) && (i + 1 < argc))
       wxlioptions = argv[++i];
+    else if((strcmp(argv[i],"-wxlisol") == 0) && (i + 2 < argc)) {
+      wxliname = argv[++i];
+      wxlisol = argv[++i];
+    }
+    else if((strcmp(argv[i],"-wxlisolopt") == 0) && (i + 1 < argc))
+      wxlisoloptions = argv[++i];
     else if((strcmp(argv[i],"-rbas") == 0) && (i + 1 < argc))
       rbasname = argv[++i];
     else if((strcmp(argv[i],"-wbas") == 0) && (i + 1 < argc))
@@ -650,6 +680,11 @@ int main(int argc, char *argv[])
       epsel = atof(argv[++i]);
     else if(strcmp(argv[i],"-parse_only") == 0)
       parse_only = TRUE;
+    else
+      ok = TRUE;
+
+    if(!ok)
+      ;
     else if(strcmp(argv[i],"-presolverow") == 0)
       or_value(&do_presolve, PRESOLVE_ROWS);
     else if(strcmp(argv[i],"-presolvecol") == 0)
@@ -702,6 +737,10 @@ int main(int argc, char *argv[])
       i++;
     else if((strcmp(argv[i],"-wparopt") == 0) && (i + 1 < argc))
       i++;
+	else if(strcmp(argv[i],"-o0") == 0)
+	  obj_in_basis = FALSE;
+	else if(strcmp(argv[i],"-o1") == 0)
+	  obj_in_basis = TRUE;
     else if(fpin == stdin) {
       filen = argv[i];
       if(*filen == '<')
@@ -792,6 +831,9 @@ int main(int argc, char *argv[])
       set_minim(lp);
   }
 
+  if (obj_in_basis != -1)
+    set_obj_in_basis(lp, obj_in_basis);
+
   if(noint) { /* remove integer conditions */
     for(i = get_Ncolumns(lp); i >= 1; i--) {
       if(is_SOS_var(lp, i)) {
@@ -805,7 +847,7 @@ int main(int argc, char *argv[])
   }
 
   if(!write_model_after)
-    write_model(lp, wlp, wmps, wfmps, wxli, wxliname, wxlioptions);
+    write_model(lp, wlp, wmps, wfmps, wxli, NULL, wxliname, wxlioptions);
 
   if(parse_only) {
     if(!write_model_after) {
@@ -917,6 +959,55 @@ int main(int argc, char *argv[])
     set_scalelimit(lp, scaleloop);
   }
 
+  if(guessbasis != NULL) {
+    REAL *guessvector, a;
+	int *basisvector;
+	int Nrows = get_Nrows(lp);
+	int Ncolumns = get_Ncolumns(lp);
+	int col;
+	char buf[50], *ptr;
+	FILE *fp;
+
+	if ((fp = fopen(guessbasis, "r")) != NULL) {
+	  guessvector = (REAL *) calloc(1+Ncolumns, sizeof(*guessvector));
+	  basisvector = (int *) malloc((1+Nrows+Ncolumns)*sizeof(*basisvector));
+	  if ((guessvector != NULL) && (basisvector != NULL)) {
+	    while ((!feof(fp)) && (fgets(buf, sizeof(buf), fp) != NULL)) {
+  		  ptr = strrchr(buf, ':');
+		  if (ptr == NULL) {
+			printf("Mallformed line: %s\n", buf);
+		  }
+		  else {
+		    a = atof(ptr + 1);
+		    while ((ptr > buf) && (isspace(ptr[-1])))
+			  ptr--;
+		    *ptr = 0;
+			col = get_nameindex(lp, buf, FALSE);
+			if (col < 1)
+			  printf("guess_basis: Unknown variable name %s\n", buf);
+			else
+		      guessvector[col] = a;
+		  }
+		}
+	    if (guess_basis(lp, guessvector, basisvector)) {
+	      if (!set_basis(lp, basisvector, TRUE))
+		    printf("Unable to set guessed basis.\n");
+	    }
+	    else
+	      printf("Unable to guess basis from provided variables.\n");
+	  }
+	  else
+	    printf("guess_basis: Out of memory.\n");
+	  if (basisvector != NULL)
+	    free(basisvector);
+	  if (guessvector != NULL)
+	    free(guessvector);
+	  fclose(fp);
+	}
+	else
+	  printf("Unable to open file %s\n", guessbasis);
+  }
+
   if(rbasname != NULL)
     if(!read_basis(lp, rbasname, NULL)) {
       fprintf(stderr, "Unable to read basis file.\n");
@@ -931,7 +1022,9 @@ int main(int argc, char *argv[])
       fprintf(stderr, "Unable to write basis file.\n");
 
   if(write_model_after)
-    write_model(lp, wlp, wmps, wfmps, wxli, wxliname, wxlioptions);
+    write_model(lp, wlp, wmps, wfmps, wxli, NULL, wxliname, wxlioptions);
+
+  write_model(lp, NULL, NULL, NULL, NULL, wxlisol, wxliname, wxlisoloptions);
 
   if(PRINT_SOLUTION >= 6)
     print_scales(lp);
@@ -961,11 +1054,16 @@ int main(int argc, char *argv[])
 
   switch(result) {
   case SUBOPTIMAL:
-    if (PRINT_SOLUTION >= 1)
-      printf("Suboptimal solution\n");
+  case PRESOLVED:
   case OPTIMAL:
   case PROCBREAK:
   case FEASFOUND:
+    if ((result == SUBOPTIMAL) && (PRINT_SOLUTION >= 1))
+      printf("Suboptimal solution\n");
+
+    if (result == PRESOLVED)
+      printf("Presolved solution\n");
+
     if (PRINT_SOLUTION >= 1)
       print_objective(lp);
 
